@@ -94,7 +94,7 @@ fn get_input(rec: &Record, col: &Option<Expr>) -> Result<String, EvalError> {
     match col {
         Some(expr) => {
             let res: String = expr.eval(&rec.data)?;
-            Ok(res.to_string())
+            Ok(res)
         }
         None => Ok(rec.raw.clone()),
     }
@@ -232,10 +232,16 @@ pub enum Expr {
     BoolUnary(UnaryExpr<BoolUnaryExpr>),
     Comparison(BinaryExpr<BoolExpr>),
     Arithmetic(BinaryExpr<ArithmeticExpr>),
+    Logical(BinaryExpr<LogicalExpr>),
     FunctionCall {
         func: &'static funcs::FunctionContainer,
         args: Vec<Expr>,
     },
+    IfOp {
+        cond: Box<Expr>,
+        value_if_true: Box<Expr>,
+        value_if_false: Box<Expr>,
+    },
     Value(&'static data::Value),
 }
 
@@ -284,6 +290,12 @@ pub enum ArithmeticExpr {
     Divide,
 }
 
+#[derive(Clone, Debug)]
+pub enum LogicalExpr {
+    And,
+    Or,
+}
+
 impl<T: Copy + Send + Sync> Evaluatable<T> for T {
     fn eval(&self, _record: &HashMap<String, data::Value>) -> Result<T, EvalError> {
         Ok(*self)
@@ -319,6 +331,28 @@ impl Evaluatable<data::Value> for BinaryExpr<ArithmeticExpr> {
     }
 }
 
+impl Evaluatable<data::Value> for BinaryExpr<LogicalExpr> {
+    fn eval(&self, record: &HashMap<String, data::Value>) -> Result<data::Value, EvalError> {
+        let l: bool = self.left.eval(record)?;
+        match self.operator {
+            LogicalExpr::And => {
+                if l {
+                    self.right.eval(record)
+                } else {
+                    Ok(data::Value::Bool(false))
+                }
+            }
+            LogicalExpr::Or => {
+                if l {
+                    Ok(data::Value::Bool(true))
+                } else {
+                    self.right.eval(record)
+                }
+            }
+        }
+    }
+}
+
 impl Evaluatable<bool> for UnaryExpr<BoolUnaryExpr> {
     fn eval(&self, record: &HashMap<String, data::Value>) -> Result<bool, EvalError> {
         let bool_res: data::Value = self.operand.eval(record)?;
@@ -402,12 +436,26 @@ impl Evaluatable<data::Value> for Expr {
                 Ok(data::Value::from_bool(bool_res))
             }
             Expr::Arithmetic(ref binary_expr) => binary_expr.eval(record),
-            Expr::FunctionCall { ref func, ref args } => {
+            Expr::Logical(ref logical_expr) => logical_expr.eval(record),
+            Expr::FunctionCall { func, ref args } => {
                 let evaluated_args: Result<Vec<data::Value>, EvalError> =
-                    args.into_iter().map(|expr| expr.eval(record)).collect();
+                    args.iter().map(|expr| expr.eval(record)).collect();
 
                 func.eval_func(&evaluated_args?)
             }
+            Expr::IfOp {
+                ref cond,
+                ref value_if_true,
+                ref value_if_false,
+            } => {
+                let evaluated_cond: bool = (*cond).eval(record)?;
+
+                if evaluated_cond {
+                    (*value_if_true).eval(record)
+                } else {
+                    (*value_if_false).eval(record)
+                }
+            }
             Expr::Value(v) => Ok(v.clone()),
         }
     }
@@ -525,7 +573,7 @@ impl CountDistinct {
 impl AggregateFunction for CountDistinct {
     fn process(&mut self, rec: &Data) -> Result<(), EvalError> {
         let value: data::Value = self.column.eval(rec)?;
-        self.state.insert(value.clone());
+        self.state.insert(value);
         Ok(())
     }
 
@@ -786,7 +834,7 @@ impl MultiGrouper {
     fn process_map(&mut self, data: &Data) {
         let key_values = self.key_cols.iter().map(|expr| expr.eval(data));
         let key_columns: Vec<data::Value> = key_values
-            .map(|value_res| value_res.unwrap_or_else(|_| data::Value::None))
+            .map(|value_res| value_res.unwrap_or(data::Value::None))
             .collect();
         let agg_col = &self.agg_col;
         let row = self.state.entry(key_columns).or_insert_with(|| {
@@ -1040,7 +1088,7 @@ impl UnaryPreAggFunction for Timeslice {
                 Ok(Some(rec))
             }
             _ => Err(EvalError::ExpectedDate {
-                found: format!("{}", inp.to_string()),
+                found: inp.to_string(),
             }),
         }
     }
@@ -1144,7 +1192,7 @@ impl UnaryPreAggFunction for ParseJson {
                 JsonValue::Bool(b) => data::Value::Bool(b),
                 JsonValue::Object(map) => data::Value::Obj(
                     map.into_iter()
-                        .map(|(k, v)| (k.to_string(), json_to_value(v)))
+                        .map(|(k, v)| (k, json_to_value(v)))
                         .collect::<HashMap<String, data::Value>>()
                         .into(),
                 ),
@@ -1194,7 +1242,7 @@ impl UnaryPreAggFunction for ParseLogfmt {
             let inp = get_input(&rec, &self.input_column)?;
             // Record includes the trailing newline, while logfmt considers that part of the
             // message if present. Trim any trailing whitespace.
-            logfmt::parse(&inp.trim_end())
+            logfmt::parse(inp.trim_end())
         };
         let res = {
             pairs.into_iter().fold(rec, |record, pair| match pair.val {
